/**
 * App Window
 */
@define-color bg_color #41403C;
@define-color window_bg @bg_color;
@define-color window_border_color #3b3b3b;
@define-color prefs_backgroud #aaa;

/**
 * Input
 */
@define-color selected_bg_color #DA481E;
@define-color selected_fg_color lighter(#ffcbbd);
@define-color input_color #ddd;
@define-color caret_color darker(@input_color);

/**
 * Result items
 */
@define-color item_name #ddd;
@define-color item_text #999;
@define-color item_box_selected #d85731;
@define-color item_text_selected #fff;
@define-color item_name_selected #fefefe;
@define-color item_shortcut_color #777;
@define-color item_shortcut_shadow darker(@bg_color);
@define-color item_shortcut_color_sel #ffcbbd;
@define-color item_shortcut_shadow_sel darker(@item_box_selected);

.app {
    background-color: @window_bg;
    border-color: @window_border_color;
}

.input {
    color: @input_color;
}

/**
 * Selected text in input
 */
.input *:selected,
.input *:focus,
*:selected:focus {
    background-color: alpha (@selected_bg_color, 0.9);
    color: @selected_fg_color;
}

.item-text {
    color: @item_text;
}
.item-name {
    color: @item_name;
}

.selected.item-box {
    background-color: @item_box_selected;
    border-left: 1px solid @window_border_color;
    border-right: 1px solid @window_border_color;
}
.selected.item-box .item-text {
    color: @item_text_selected;
}
.selected.item-box .item-name {
    color: @item_name_selected;
}
.item-shortcut {
    color: @item_shortcut_color;
    text-shadow: 1px 1px 1px @item_shortcut_shadow;
}
.selected.item-box .item-shortcut {
    color: @item_shortcut_color_sel;
    text-shadow: 1px 1px 1px @item_shortcut_shadow_sel;
}

.prefs-btn {
    opacity: 0.8;
}
.prefs-btn:hover {
    background-color: @prefs_backgroud;
}

.no-window-shadow {
    margin: -20px;
}
